
#include "$(appName).h"
#include "animator.h"
#include "conf.h"

#define THRESHOLD 0.0000001

typedef struct {
	Evas_Object *c_obj;
	Evas_Object *n_obj;
	int x;
	int y;
	int w;
	int h;
} animation_data_t;



/*============================== EFL OBJECTS CALLBACKS ======================*/
static void
__transit_current_del_cb(void *data, Elm_Transit *transit);
static Eina_Bool
__resize_animator_timeline_cb(void *data, double pos);
static Eina_Bool
__fade_animator_timeline_cb(void *data, double pos);
static void
__split_anim_done_cb(void *data, Evas_Object *obj, const char *emission,
	const char *source);

/*============================== HELPER FUNCTIONS ===========================*/
static void
__start_wipe_animation(Evas_Object *c_page, Elm_Transit_Effect_Wipe_Dir dir);
static void
__start_resize_animation(Evas_Object *c_page, Evas_Object *n_page);
static void
__start_split_animation(Evas_Object *c_page, Evas_Object *n_page);
static void
__start_fade_animation(Evas_Object *c_page, Evas_Object *n_page);

/*============================== PUBLIC FUNCTIONS ===========================*/
void
animator_switch_page(Evas_Object *current_page, Evas_Object *next_page,
	animator_type_t animator_a, animator_type_t animator_c)
{
	if (animator_a == ANIMATION_TYPE_WIPE_LEFT
		&& animator_c == ANIMATION_TYPE_NONE) {
		__start_wipe_animation(current_page,
			ELM_TRANSIT_EFFECT_WIPE_DIR_LEFT);
	} else if (animator_a == ANIMATION_TYPE_NONE
		&& animator_c == ANIMATION_TYPE_WIPE_RIGHT) {
		__start_wipe_animation(current_page,
			ELM_TRANSIT_EFFECT_WIPE_DIR_RIGHT);
	} else if ((animator_a == ANIMATION_TYPE_RESIZE
		&& animator_c == ANIMATION_TYPE_NONE)
		|| (animator_a == ANIMATION_TYPE_NONE
		&& animator_c == ANIMATION_TYPE_RESIZE)) {
		__start_resize_animation(current_page, next_page);
	} else if ((animator_a == ANIMATION_TYPE_SPLIT
		&& animator_c == ANIMATION_TYPE_NONE)
		|| (animator_c == ANIMATION_TYPE_SPLIT
		&& animator_a == ANIMATION_TYPE_NONE)) {
		__start_split_animation(current_page, next_page);
	} else if ((animator_a == ANIMATION_TYPE_FADE
		&& animator_c == ANIMATION_TYPE_NONE)
		|| (animator_c == ANIMATION_TYPE_FADE
		&& animator_a == ANIMATION_TYPE_NONE)) {
		__start_fade_animation(current_page, next_page);
	}
}

/*============================== EFL OBJECTS CALLBACKS ======================*/
static void
__transit_current_del_cb(void *data, Elm_Transit *transit)
{
	evas_object_hide((Evas_Object *) data);
}

static Eina_Bool
__resize_animator_timeline_cb(void *data, double pos)
{
	double frame = 1.0 - pos;
	animation_data_t *anim_data = (animation_data_t *) data;
	double new_c_x = -1.0;
	double new_c_y = -1.0;
	double new_c_w = -1.0;
	double new_c_h = -1.0;
	double new_n_x = -1.0;
	double new_n_y = -1.0;
	double new_n_w = -1.0;
	double new_n_h = -1.0;

	if (!anim_data || !anim_data->c_obj || !anim_data->n_obj) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_DATA]");
		free(anim_data);
		return ECORE_CALLBACK_CANCEL;
	}

	new_c_w = anim_data->w * frame;
	new_c_h = anim_data->h * frame;
	new_c_x = (anim_data->x + (double)(anim_data->w - new_c_w)/2);
	new_c_y = (anim_data->y + (double)(anim_data->h - new_c_h)/2);

	new_n_w = anim_data->w * pos;
	new_n_h = anim_data->h * pos;
	new_n_x = (anim_data->x + (double)(anim_data->w - new_n_w)/2);
	new_n_y = (anim_data->y + (double)(anim_data->h - new_n_h)/2);

	evas_object_resize(anim_data->c_obj, new_c_w, new_c_h);
	evas_object_move(anim_data->c_obj, new_c_x, new_c_y);
	evas_object_color_set(anim_data->c_obj, 255, 255, 255, 255 * frame);

	evas_object_resize(anim_data->n_obj, new_n_w, new_n_h);
	evas_object_move(anim_data->n_obj, new_n_x, new_n_y);
	evas_object_color_set(anim_data->n_obj, 255, 255, 255, 255 * pos);

	if (fabs(pos - 1.0) < THRESHOLD) {
		evas_object_hide(anim_data->c_obj);
		evas_object_color_set(anim_data->c_obj, 255, 255, 255, 255);
		free(anim_data);
		return ECORE_CALLBACK_CANCEL;
	}

	return ECORE_CALLBACK_RENEW;
}

static void
__split_anim_done_cb(void *data, Evas_Object *obj,
				const char *emission, const char *source)
{
	evas_object_hide((Evas_Object *) data);
}

static Eina_Bool
__fade_animator_timeline_cb(void *data, double pos)
{
	double frame = 1.0 - pos;
	animation_data_t *anim_data = (animation_data_t *) data;

	if (!anim_data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA]");
		return ECORE_CALLBACK_CANCEL;
	}

	evas_object_color_set(anim_data->c_obj, 255, 255, 255, 255 * frame);
	evas_object_color_set(anim_data->n_obj, 255, 255, 255, 255 * pos);

	if (fabs(pos -  1.0) < THRESHOLD) {
		evas_object_hide(anim_data->c_obj);
		evas_object_color_set(anim_data->c_obj, 255, 255, 255, 255);
		free(anim_data);
		return ECORE_CALLBACK_CANCEL;
	}

	return ECORE_CALLBACK_RENEW;
}

/*============================== HELPER FUNCTIONS ===========================*/
static void
__start_wipe_animation(Evas_Object *c_page,
	Elm_Transit_Effect_Wipe_Dir dir)
{
	if (!c_page) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAMS]");
		return;
	}

	Elm_Transit *c_page_tranist = NULL;
	c_page_tranist = elm_transit_add();

	if (!c_page_tranist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][c_page_tranist=NULL]");
		return;
	}

	elm_transit_object_add(c_page_tranist, c_page);
	elm_transit_effect_wipe_add(c_page_tranist,
		ELM_TRANSIT_EFFECT_WIPE_TYPE_HIDE, dir);
	elm_transit_duration_set(c_page_tranist, 0.5);
	elm_transit_del_cb_set(c_page_tranist, __transit_current_del_cb, c_page);
	elm_transit_go(c_page_tranist);
}

static void
__start_resize_animation(Evas_Object *c_page, Evas_Object *n_page)
{
	Ecore_Animator *animator = NULL;
	animation_data_t *anim_data = NULL;
	int x = -1, y = -1, w = -1, h = -1;

	if (!c_page || !n_page) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAMS]");
		return;
	}

	evas_object_geometry_get(c_page, &x, &y, &w, &h);
	anim_data = (animation_data_t *) malloc(sizeof(*anim_data));
	anim_data->c_obj = c_page;
	anim_data->n_obj = n_page;
	anim_data->x = x;
	anim_data->y = y;
	anim_data->w = w;
	anim_data->h = h;

	animator = ecore_animator_timeline_add(0.5,
		__resize_animator_timeline_cb, anim_data);
}

static void
__start_split_animation(Evas_Object *c_page, Evas_Object *n_page)
{
	elm_object_signal_emit(c_page, SIGNAL_SPLIT_HORIZONAL, SIGNAL_SOURCE);
	elm_object_signal_callback_add(c_page, SIGNAL_SPLIT_ANIM_DONE,
		SIGNAL_SOURCE, __split_anim_done_cb, c_page);
}

static void
__start_fade_animation(Evas_Object *c_page, Evas_Object *n_page)
{
	Ecore_Animator *animator = NULL;
	animation_data_t *anim_data = NULL;
	anim_data = (animation_data_t *) malloc(sizeof(*anim_data));

	if (!anim_data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][anim_data=NULL]");
		return;
	}

	anim_data->c_obj = c_page;
	anim_data->n_obj = n_page;

	animator = ecore_animator_timeline_add(1.0, __fade_animator_timeline_cb,
		anim_data);
}
