#include <Elementary.h>

#include "view.h"
#include "conf.h"
#include "$(appName).h"
#include "animator.h"

typedef enum {
	PAGE_TYPE_A = 0,
	PAGE_TYPE_B,
	PAGE_TYPE_C
} page_type_t;

static char radio_names[ANIM_TYPES_CNT][ANIM_STR_LEN] = {"NO ANIMATION", "ANIMATION WIPE", "ANIMATION SPLIT", "ANIMATION FADE", "ANIMATION RESIZE"};

/*================================ STATIC FUNCTIONS DECLARATIONS ============*/
static bool
__create_win(s_view_data *view);
static bool
__create_conformant(s_view_data *view);
static bool
__create_layout(s_view_data *view);
static bool
__create_toolbar(s_view_data *view);
static bool
__create_view_page(s_view_data *view, page_type_t p_type);

static bool
__create_animation_selectors(s_view_data *view);
static bool
__fill_animation_selectors(s_view_data *view);
static Evas_Object *
__create_radio_button(s_view_data *view, Evas_Object *box, const char *name, int r_id);

static bool
__set_animations_type(s_view_data *view);

/*================================ CALLBACK FUNCTIONS DECLARATIONS ===========*/
static void
__layout_back_key_cb(void *data, Evas_Object *obj, void *event_info);
static void
__win_delete_cb(void *data, Evas_Object *obj, void *event_info);

/*Toolbar actions*/
static void
__screen_a_activate(void *data, Evas_Object *obj, void *event_info);
static void
__screen_b_activate(void *data, Evas_Object *obj, void *event_info);
static void
__screen_c_activate(void *data, Evas_Object *obj, void *event_info);

/*================================ PUBLIC FUNCTIONS DEFINITIONS =============*/
bool
view_init(s_view_data *view)
{
	if (!__create_win(view)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[create win failed]");
		return false;
	}

	if (!__create_conformant(view)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[create conformant failed]");
		return false;
	}

	if (!__create_layout(view)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[create layout failed]");
		return false;
	}

	if (!__create_toolbar(view)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[create toolbar failed]");
		return false;
	}

	return true;
}

/*================================ STATIC FUNCTIONS DEFINITIONS ==============*/
static bool
__create_win(s_view_data *view)
{
	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL]");
		return false;
	}

	view->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	if (!view->win) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view->win=NULL]");
		return false;
	}

	elm_win_conformant_set(view->win, EINA_TRUE);
	elm_win_autodel_set(view->win, EINA_TRUE);
	elm_win_indicator_mode_set(view->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(view->win, ELM_WIN_INDICATOR_OPAQUE);
	evas_object_smart_callback_add(view->win, "delete,request",
		__win_delete_cb, view);

	if (elm_win_wm_rotation_supported_get(view->win)) {
		int rots[4] = {0, 90, 180, 270};
		elm_win_wm_rotation_available_rotations_set(view->win,
			(const int *)(&rots), 4);
	}
	evas_object_show(view->win);

	return true;
}

static bool
__create_conformant(s_view_data *view)
{
	if (!view || !view->win) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL or view->win=NULL");
		return false;
	}

	view->conformant = elm_conformant_add(view->win);
	if (!view->conformant) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view->conformant=NULL]");
		return false;
	}

	evas_object_size_hint_weight_set(view->conformant, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	elm_win_resize_object_add(view->win, view->conformant);
	evas_object_show(view->conformant);

	return true;
}

static bool
__create_layout(s_view_data *view)
{
	char edje_path[PATH_MAX] = {0, };

	if (!view || !view->conformant) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL or view->conformant=NULL]");
		return false;
	}

	get_app_resource(EDJ_FILE, edje_path, (int) PATH_MAX);

	view->layout = elm_layout_add(view->win);
	if (!view->layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][layout=NULL]");
		return false;
	}

	elm_layout_file_set(view->layout, edje_path, GROUP_MAIN);
	evas_object_size_hint_weight_set(view->layout, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	eext_object_event_callback_add(view->layout, EEXT_CALLBACK_BACK,
		__layout_back_key_cb, view);
	elm_object_content_set(view->conformant, view->layout);
	evas_object_show(view->layout);
	return true;
}

static bool
__create_toolbar(s_view_data *view)
{
	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL]");
		return false;
	}

	view->toolbar = elm_toolbar_add(view->layout);
	if (!view->toolbar) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view->toolbar=NULL]");
		return false;
	}

	elm_object_style_set(view->toolbar, "tabbar");
	elm_toolbar_shrink_mode_set(view->toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_transverse_expanded_set(view->toolbar, EINA_TRUE);

	view->screen_a_indice = elm_toolbar_item_append(view->toolbar,
		NULL, "Screen A", __screen_a_activate, view);
	view->screen_b_indice = elm_toolbar_item_append(view->toolbar,
		NULL, "Screen B", __screen_b_activate, view);
	view->screen_c_indice = elm_toolbar_item_append(view->toolbar,
		NULL, "Screen C", __screen_c_activate, view);

	elm_toolbar_select_mode_set(view->toolbar,
		ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_toolbar_item_selected_set(view->screen_b_indice, EINA_TRUE);

	elm_object_part_content_set(view->layout, PART_TOOLBAR, view->toolbar);
	evas_object_size_hint_weight_set(view->toolbar, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	evas_object_show(view->toolbar);

	return true;
}

static bool
__create_view_page(s_view_data *view, page_type_t p_type)
{
	dlog_print(DLOG_INFO, LOG_TAG, "[%d]", p_type);
	char edje_path[PATH_MAX] = {0, };
	Evas_Object *page = NULL;

	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL]");
		return false;
	}

	page = elm_layout_add(view->layout);
	if (!page) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view->b_page=NULL]");
		return false;
	}

	get_app_resource(EDJ_FILE, edje_path, (int) PATH_MAX);

	switch (p_type) {
	case PAGE_TYPE_A:
		if (!elm_layout_file_set(page, edje_path, GROUP_PAGE_CUSTOM)) {
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to load edje file]");
			evas_object_del(page);
			return false;
		}
		elm_object_part_text_set(page, PART_PAGE_TITLE, "Screen-A");
		elm_object_signal_emit(page, SIGNAL_SCREEN_A_BG, SIGNAL_SOURCE);
		view->a_page = page;
		break;
	case PAGE_TYPE_B:
		if (!elm_layout_file_set(page, edje_path, GROUP_PAGE_DEFAULT)) {
			evas_object_del(page);
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to load edje file]");
			return false;
		}
		elm_object_part_text_set(page, PART_PAGE_TITLE, "Screen-B");
		view->b_page = page;
		if (!__create_animation_selectors(view)) {
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to add selectors]");
			evas_object_del(page);
			return false;
		}
		break;
	case PAGE_TYPE_C:
		if (!elm_layout_file_set(page, edje_path, GROUP_PAGE_CUSTOM)) {
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to load edje file]");
			evas_object_del(page);
			return false;
		}
		elm_object_part_text_set(page, PART_PAGE_TITLE, "Screen-C");
		elm_object_signal_emit(page, SIGNAL_SCREEN_C_BG, SIGNAL_SOURCE);
		view->c_page = page;
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][unknown p_type]");
		evas_object_del(page);
		return false;
	}

	evas_object_size_hint_weight_set(page, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	evas_object_show(page);
	return true;
}

static bool
__create_animation_selectors(s_view_data *view)
{
	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view=NULL]");
		return false;
	}

	if (view->group_1_box || view->group_2_box)
		return true;

	view->group_1_box = elm_box_add(view->b_page);
	if (!view->group_1_box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to create elm_box]");
		return false;
	}

	elm_object_part_content_set(view->b_page, PART_PAGE_DEFAULT_ANIM_TYPE_A, view->group_1_box);
	evas_object_size_hint_weight_set(view->group_1_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(view->group_1_box);
	elm_box_align_set(view->group_1_box, 0.5, 0.1);

	view->group_2_box = elm_box_add(view->b_page);
	if (!view->group_2_box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to create elm_box]");
		return false;
	}

	elm_object_part_content_set(view->b_page, PART_PAGE_DEFAULT_ANIM_TYPE_C, view->group_2_box);
	evas_object_size_hint_weight_set(view->group_2_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(view->group_2_box);
	elm_box_align_set(view->group_2_box, 0.5, 0.1);

	if (!__fill_animation_selectors(view)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to fill animation selectors]");
		return false;
	}

	return true;
}

static Evas_Object *
__create_radio_button(s_view_data *view, Evas_Object *box, const char *name, int r_id)
{
	if (!view || !name || !box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view == NULL || name==NULL || box == NULL]");
		return NULL;
	}

	Evas_Object *radio = elm_radio_add(box);
	if (!radio) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to create elm radio]");
		return NULL;
	}

	elm_object_part_text_set(radio, "default", name);
	elm_radio_state_value_set(radio, r_id);
	evas_object_size_hint_align_set(radio, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(radio);
	elm_box_pack_end(box, radio);

	return radio;
}

static bool
__fill_animation_selectors(s_view_data *view)
{
	Evas_Object *radio = NULL;

	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view == NULL]");
		return false;
	}

	int r_id_1 = 0;
	int r_id_2 = 0;
	int i = 0;

	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view=NULL]");
		return false;
	}

	if (!view->group_1_box || !view->group_2_box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view->group_1_box='%p'][view->group_2_box='%p']",
				view->group_1_box, view->group_2_box);
		return false;
	}

	view->radio_gr_1 = __create_radio_button(view, view->group_1_box, radio_names[0], r_id_1++);
	if (!view->radio_gr_1)
		return false;

	view->radio_gr_2 = __create_radio_button(view, view->group_2_box, radio_names[0], r_id_2++);
	if (!view->radio_gr_2)
		return false;

	for (i = 1; i < ANIM_TYPES_CNT; i++) {
		radio = __create_radio_button(view, view->group_1_box, radio_names[i], r_id_1++);
		if (!radio)
			return false;
		elm_radio_group_add(radio, view->radio_gr_1);

		radio = __create_radio_button(view, view->group_2_box, radio_names[i], r_id_2++);
		if (!radio)
			return false;

		elm_radio_group_add(radio, view->radio_gr_2);
	}

	elm_radio_value_pointer_set(view->radio_gr_1, &(view->actual_indice_scr_a));
	elm_radio_value_pointer_set(view->radio_gr_2, &(view->actual_indice_scr_c));

	return true;
}

static bool
__set_animations_type(s_view_data *view) {
	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][view == NULL]");
		return false;
	}

	switch (view->actual_indice_scr_a) {
	case 0:
		view->animator_scr_a = ANIMATION_TYPE_NONE;
		break;
	case 1:
		view->animator_scr_a = ANIMATION_TYPE_WIPE_LEFT;
		break;
	case 2:
		view->animator_scr_a = ANIMATION_TYPE_SPLIT;
		break;
	case 3:
		view->animator_scr_a = ANIMATION_TYPE_FADE;
		break;
	case 4:
		view->animator_scr_a = ANIMATION_TYPE_RESIZE;
		break;
	default:
		break;
	}

	switch (view->actual_indice_scr_c) {
	case 0:
		view->animator_scr_c = ANIMATION_TYPE_NONE;
		break;
	case 1:
		view->animator_scr_c = ANIMATION_TYPE_WIPE_RIGHT;
		break;
	case 2:
		view->animator_scr_c = ANIMATION_TYPE_SPLIT;
		break;
	case 3:
		view->animator_scr_c = ANIMATION_TYPE_FADE;
		break;
	case 4:
		view->animator_scr_c = ANIMATION_TYPE_RESIZE;
		break;
	default:
		break;
	}

	return true;
}

/*================================ CALLBACK FUNCTIONS DEFINITIONS ===========*/
static void
__layout_back_key_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_view_data *view = (s_view_data *) data;
	elm_win_lower(view->win);
}

static void
__win_delete_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static void
__screen_a_activate(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *current_page = NULL;
	animator_type_t tmp = ANIMATION_TYPE_NONE;
	s_view_data *view = (s_view_data *) data;

	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA]");
		return;
	}

	if (view->active_view == ACTIVE_VIEW_TYPE_SCR_A)
		return;

	if (!view->a_page && (!__create_view_page(view, PAGE_TYPE_A))) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to create the view page]");
		return;
	}

	current_page = elm_object_part_content_unset(view->layout, PART_PAGE);
	elm_object_part_content_set(view->layout, PART_PAGE, view->a_page);
	evas_object_show(view->a_page);

	__set_animations_type(view);

	if (view->animator_scr_a != ANIMATION_TYPE_NONE) {
		tmp = view->animator_scr_c;
		view->animator_scr_c = ANIMATION_TYPE_NONE;

		animator_switch_page(current_page, view->a_page,
			view->animator_scr_a, view->animator_scr_c);
		view->animator_scr_c = tmp;
	} else {
		evas_object_hide(current_page);
	}

	view->active_view = ACTIVE_VIEW_TYPE_SCR_A;
}

static void
__screen_b_activate(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *current_page = NULL;
	s_view_data *view = (s_view_data *) data;
	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA]");
		return;
	}

	if (view->active_view == ACTIVE_VIEW_TYPE_SCR_B)
		return;

	if (!view->b_page && (!__create_view_page(view, PAGE_TYPE_B))) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to create the view page]");
		return;
	}

	current_page = elm_object_part_content_unset(view->layout, PART_PAGE);
	evas_object_hide(current_page);
	__set_animations_type(view);

	elm_object_part_content_set(view->layout, PART_PAGE, view->b_page);
	evas_object_show(view->b_page);

	view->active_view = ACTIVE_VIEW_TYPE_SCR_B;
}

static void
__screen_c_activate(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *current_page = NULL;
	animator_type_t tmp = ANIMATION_TYPE_NONE;
	s_view_data *view = (s_view_data *) data;

	if (!view) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA]");
		return;
	}

	if (view->active_view == ACTIVE_VIEW_TYPE_SCR_C)
		return;

	if (!view->c_page && (!__create_view_page(view, PAGE_TYPE_C))) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to create the view page]");
		return;
	}

	current_page = elm_object_part_content_unset(view->layout, PART_PAGE);
	elm_object_part_content_set(view->layout, PART_PAGE, view->c_page);
	evas_object_show(view->c_page);
	__set_animations_type(view);

	if (view->animator_scr_c != ANIMATION_TYPE_NONE) {
		tmp = view->animator_scr_a;
		view->animator_scr_a = ANIMATION_TYPE_NONE;

		animator_switch_page(current_page, view->c_page,
			view->animator_scr_a, view->animator_scr_c);
		view->animator_scr_a = tmp;
	} else {
		evas_object_hide(current_page);
	}

	view->active_view = ACTIVE_VIEW_TYPE_SCR_C;
}
